setClass("RPPADesign",
         representation=list(
           layout="data.frame",
           alias="data.frame",
           sampleMap="character",
           controls="list"
           ))

RPPADesign <- function(raw,
                       steps=NULL,
                       series=NULL,
                       grouping=c("byRow", "byCol", "bySample", "blockSample"),
                       ordering=c("decreasing", "increasing"),
                       alias=NULL, center=FALSE, controls=list()) {
  if (is.null(alias)) {
    alias <- data.frame(Alias=levels(raw@data$Sample), Sample=levels(raw@data$Sample))
  }
  if (inherits(raw, "RPPA")) raw <- raw@data
  temp <- data.frame(raw[, c("Main.Row", "Main.Col", "Sub.Row", "Sub.Col", "Sample")])
  if (is.null(steps) & is.null(series)) {
    grouping <- match.arg(grouping)
    ordering <- match.arg(ordering)
    steps <- rep(NA, nrow(raw))
    series <- rep(NA, nrow(raw))
    if (grouping == "byRow") {
      series <- factor(paste("Series", temp$Main.Row, temp$Main.Col,
                             temp$Sub.Row, sep='.'))
      if (ordering == "increasing") {
        steps <- temp$Sub.Col - 1
      } else { # ordering == "decreasing"
        steps <- max(temp$Sub.Col) - temp$Sub.Col
      }
    } else if (grouping == "byCol") {
      series <- factor(paste("Series", temp$Main.Row, temp$Main.Col,
                             temp$Sub.Col, sep='.'))
      if (ordering == "increasing") {
        steps <- temp$Sub.Row - 1
      } else { # ordering == "decreasing"
        steps <- max(temp$Sub.Row) - temp$Sub.Row
      }
    } else if (grouping == "bySample") {
      series <- temp$Sample
      for (sam in levels(temp$Sample)) {
        where <- temp$Sample == sam
        n <- sum(where)
        if (ordering == "increasing") {
          steps[where] <- -1 + (1:n)
        } else { # ordering == "decreasing"
          steps[where] <- n - (1:n)
        }
      } # end for(sam)
    } else if (grouping == "blockSample") {
      attach(temp)
      series <- factor(paste(as.character(Sample), Main.Row, Main.Col, sep='.'))
      detach()
      for (sam in levels(series)) {
        where <- series == sam
        n <- sum(where)
        if (ordering == "increasing") {
          steps[where] <- -1 + (1:n)
        } else { # ordering == "decreasing"
          steps[where] <- n - (1:n)
        }
      } # end for(sam)      
    } # end if grouping
  } else if (is.null(steps) | is.null(series)) {
    stop("You must supply both 'steps' and 'series' if you supply either one")
  } else { # both series and steps supplied
    if(length(steps) != nrow(temp) | length(series) != nrow(temp)) {
      stop("lengths do not match")
    }
  }# end if(is.null(steps))
  if (center) {
    for (ser in levels(series)) {
      where <- series == ser
      steps[where] <- steps[where] - median(steps[where])
    }
  }
  if(any(is.na(steps))) warning("Some dilution steps have not been specified")
  d <- as.character(temp$Sample)
  dd <- which(is.element(d, controls))
  steps[dd] <- 0
  temp$Steps <- steps
  temp$Series <- series
  sampleMap <- as.vector(tapply(as.character(temp$Sample), list(series), function(x) x[[1]]))
  names(sampleMap) <- levels(series)
  new("RPPADesign",
      layout=temp,
      alias=alias,
      sampleMap=sampleMap,
      controls=controls)
}


setMethod("image", "RPPADesign",
          function(x, colorbar=FALSE, col=terrain.colors(256), ...) {
  # figure out how to make "geographic" pictures
  temp <- x@layout
  my <- max(temp$Main.Row) * max(temp$Sub.Row)
  mx <- max(temp$Main.Col) * max(temp$Sub.Col)
  yspot <- 1+my-(max(temp$Sub.Row)*(temp$Main.Row-1) + temp$Sub.Row)
  xspot <- max(temp$Sub.Col)*(temp$Main.Col-1) + temp$Sub.Col
  geo.steps <- tapply(temp$Steps, list(xspot, yspot), mean)
  if (colorbar) {
    ##layout(matrix(c(2, 1), nrow=1), widths=c(10,1))
    ##opar <- par(mai=c(1, 0.5, 1, 0.1))
    ##on.exit(par(opar))
    ##on.exit(layout(1))
    ##image(1, seq(min(geo.steps, na.rm=TRUE),
    ##             max(geo.steps, na.rm=TRUE), length=256),
    ##      matrix(1:256, nrow=1), col=col,
    ##      xaxt='n', xlab='', ylab='')
    
    ## first, get the size of the plotting region
    ## in relative units
    startPlt <- par()$plt;
    
    ## We're only going to partition things on the
    ## x-axis, so only the first 2 coordinates are of
    ## interest. Define the boundaries for the two panels
    ## so that a 10/1 width ratio is attained. 
    imagePlt    <- startPlt;
    colorbarPlt <- startPlt;
    startWidth  <- startPlt[2] - startPlt[1];

    imagePlt[2]    <- startPlt[1] + (10/12)*startWidth;
    colorbarPlt[1] <- startPlt[2] - ( 1/12)*startWidth;

    ## draw the colorbar
    par(plt=colorbarPlt);
    image(1, seq(min(geo.steps, na.rm=TRUE),
                 max(geo.steps, na.rm=TRUE), length=256),
          matrix(1:256, nrow=1), col=col,
          xaxt='n', yaxt='n', xlab='', ylab='')
    axis(4); ## put labeling at right

    ## set things up to draw the main image and
    ## revert back for the next figure
    par(plt=imagePlt, new=TRUE);
    on.exit(par(plt=startPlt));
  }
  image(1:mx, 1:my, geo.steps, col=col, ...)
  abline(h = 0.5 + seq(0, my, length=1+max(temp$Main.Row)))
  abline(v = 0.5 + seq(0, mx, length=1+max(temp$Main.Col)))
  invisible(geo.steps)
})

setMethod("summary", "RPPADesign", function(object, ...) {
  cat("An RPPA design object with controls:\n")
  print(unlist(object@controls))
  cat("\n")
  summary(object@layout)
})

.controlVector <- function(design) {
  sample <- as.character(design@layout$Sample)
  temp <- rep(FALSE, length(unique(sample)))
  names(temp) <- unique(sample)
  temp[unlist(design@controls)] <- TRUE
  temp[sample]
}
  
seriesNames <- function(design) {
  if (!inherits(design, "RPPADesign"))
    stop("Invalid design class")
  isControl <- .controlVector(design)
  series <- as.character(design@layout$Series[!isControl])
  unique(series)
}

getSteps <- function(design) {
  isControl <- .controlVector(design)
  design@layout$Steps[!isControl]
}

setMethod("names", "RPPADesign", function(x) {
  isControl <- .controlVector(x)
  as.character(x@layout$Series[!isControl])
})

if(FALSE) {
  path <- "../inst/rppaTumorData"
  erk2 <- RPPA("ERK2.txt", path=path)
  design <- RPPADesign(erk2, grouping="blockSample", center=TRUE)
  image(design)
  summary(design)
  design <- RPPADesign(erk2, grouping="blockSample",
                       controls=list("neg con", "pos con"))
  image(design)
  summary(design)
  rm(path, erk2, design)
}

